@@ -41,7 +41,7 @@ import java.util.Properties;
  * connection-accepting server.
  *
  * @author Gennis Emerson
- * @version $Revision: 1.33 $
+ * @version $Revision: 1.34 $
  */
 /* The handleMessage methods may throw only SQLExceptions and 
  * RuntimeExceptions, the first if they can recover, the second
@@ -996,6 +996,11 @@ class Connection implements Runnable
     BerObject handleRequest(StatementDestroyRequest aRequest)
         throws DbdException
     {
+        /*
+        if (true)
+            throw new DbdException(DbdException.gGENERIC_EXCEPTION,
+                new String[] { "Fake error in server statement destroy" });
+        */
         // Note: we want to call mStatementTable.remove, so don't use
         // mGetStatementHolder.
         int handle = aRequest.getHandle();
