@@ -24,13 +24,10 @@ package com.vizdom.util;
  * A public toolbox of assorted methods that don't belong anywhere else.
  *
  * @author John Lacey
- * @version $Revision: 1.19 $
+ * @version $Revision: 1.26 $
  */
 public class Toolbox
 {
-    /** Instances of Toolbox are not permitted. */
-    private Toolbox() {}
-
     /** 
      * Checks whether a string is uppercase.
      *
@@ -48,6 +45,7 @@ public class Toolbox
         return true;
     }
 
+
     /** 
      * Splits a string into substrings and returns the pieces as the
      * elements of a string array. The string is split on a specified
@@ -85,6 +83,7 @@ public class Toolbox
         return strings;
     }
 
+
     /**
      * Splits a string at each occurrence of a given character. Two
      * delimiter characters in a row will cause the splitting to stop;
@@ -237,6 +236,7 @@ public class Toolbox
         return toString(aList, String.valueOf(aDelimiter));
     }
 
+
     /**
      * Joins the elements of the given array into a string, 
      * separated by the delimiter character.
@@ -257,6 +257,7 @@ public class Toolbox
             String.valueOf(aSentinel));
     }
 
+
     /**
      * Joins the elements of the given array into a string, 
      * separated by the delimiter character.
@@ -274,6 +275,7 @@ public class Toolbox
         return toString(aList, String.valueOf(aDelimiter));
     }
 
+
     /**
      * Joins the elements of the given array into a string, 
      * separated by the delimiter character.
@@ -294,6 +296,7 @@ public class Toolbox
             String.valueOf(aSentinel));
     }
 
+
     /**
      * Joins the elements of the given array into a string, 
      * separated by the delimiter string.
@@ -310,6 +313,7 @@ public class Toolbox
         return toString(aList, aDelimiter, "");
     }
 
+
     /**
      * Joins the elements of the given array into a string, 
      * separated by the delimiter string.
@@ -345,6 +349,7 @@ public class Toolbox
         }
         return value;
     }
+
     
     /**
      * Joins the elements of the given array into a string, 
@@ -362,6 +367,7 @@ public class Toolbox
         return toString(aList, aDelimiter, "");
     }
 
+
     /**
      * Joins the elements of the given array into a string, 
      * separated by the delimiter string.
@@ -413,19 +419,8 @@ public class Toolbox
         int length = aString.length();
         if (length > 0 && aString.charAt(length - 1) == '\u0000')
         {
-            StringBuffer buffer = new StringBuffer(length);
-            buffer.append(aString);
             // ??? remove all trailing null characters, or just one?
-            /*
-            do
-            {
-            */
-                buffer.setLength(--length);
-            /*
-            }
-            while (buffer.charAt(length - 1) == '\u0000');
-            */
-            return buffer.toString();
+            return aString.substring(0, length - 1);
         }
         else
             return aString;
@@ -441,20 +436,10 @@ public class Toolbox
      */
     public static final String toCString(String aString)
     {
-        int length = aString.length();
-        /*
-        if (length > 0 && aString.charAt(length - 1) == '\u0000')
-            return aString;
-        else
-        {
-        */
-            StringBuffer buffer = new StringBuffer(length + 1);
-            buffer.append(aString);
-            buffer.append('\u0000');
-            return buffer.toString();
-        /*
-        }
-        */
+        StringBuffer buffer = new StringBuffer(aString.length() + 1);
+        buffer.append(aString);
+        buffer.append('\u0000');
+        return buffer.toString();
     }
 
 
@@ -474,7 +459,7 @@ public class Toolbox
     public static final byte[] hexToBytes(String aHexString)
     {
         int nybbleCount = aHexString.length();
-        Debug.assert((nybbleCount % 2) == 0);
+        Debug.assertTrue((nybbleCount % 2) == 0);
         byte[] bytes = new byte[nybbleCount / 2];
         for (int i = 0; i < nybbleCount - 1; i += 2)
         {
@@ -485,6 +470,7 @@ public class Toolbox
         return bytes;
     }
 
+
     /**
      * Converts each byte to a pair of hexadecimal characters.
      * <p>
@@ -516,13 +502,38 @@ public class Toolbox
                 hexIndex += 2;
             }
             if (Debug.ASSERT)
-                Debug.assert(hexIndex == hexChars.length);
+                Debug.assertTrue(hexIndex == hexChars.length);
             hexString = new String(hexChars);
         }
         return hexString;
     }
 
 
+    /**
+     * Loads properties from the given input stream and returns the
+     * corresponding properties object.
+     *
+     * @param in an input stream whose content is in properties file
+     *     format; this stream will be closed
+     * @return a Properties object constructed from the given input stream
+     * @exception java.io.IOException if an error occurs reading the file
+     */
+    private static java.util.Properties gGetProperties(java.io.InputStream in)
+        throws java.io.IOException 
+    {
+        try
+        {
+            java.util.Properties properties = new java.util.Properties();
+            properties.load(in);
+            return properties;
+        }
+        finally
+        {
+            try { in.close(); } catch (Exception e) {}
+        }
+    }
+
+
     /**
      * Loads a properties file with the given name and returns the
      * corresponding properties object. The file must be located
@@ -531,8 +542,8 @@ public class Toolbox
      *
      * @param aName the base portion (no extension) of the properties filename
      * @return a Properties object constructed from the given file
-     * @exception IOException if the properties file is not found, or an 
-     *     error occurs reading the file,
+     * @exception java.io.IOException if the properties file is not
+     *     found, or an error occurs reading the file,
      */
     public static final java.util.Properties getProperties(String aName) 
         throws java.io.IOException 
@@ -541,16 +552,30 @@ public class Toolbox
         java.io.InputStream is = ClassLoader.getSystemResourceAsStream(name);
         if (is == null) 
             throw new java.io.FileNotFoundException(name);
-        try
-        {
-            java.util.Properties properties = new java.util.Properties();
-            properties.load(is);
-            return properties;
-        }
-        finally
-        {
-            try { is.close(); } catch (Exception e) {}
-        }
+        return gGetProperties(is);
+    }
+
+
+    /**
+     * Loads a properties file from the give pathname and returns the
+     * corresponding properties object.
+     *
+     * @param aFile the <code>java.io.File</code> specifying the file's path
+     * @return a Properties object constructed from the given file
+     * @exception java.io.IOException if the properties file is not
+     *     found, or an error occurs reading the file,
+     */
+    public static final java.util.Properties getProperties(java.io.File aFile) 
+        throws java.io.IOException 
+    {
+        java.io.FileInputStream is = new java.io.FileInputStream(aFile);
+        return gGetProperties(is);
+    }
+
+
+    /** Instances of Toolbox are not permitted. */
+    private Toolbox()
+    {
+        throw new UnreachableCodeException();
     }
 }
-
